using UnityEngine;
using System.Collections;
using System.Collections.Generic;
using Leap;

public static class LeapGUI {
	static Controller m_leapController = null;
	static Dictionary<int, bool> m_pressCache = new Dictionary<int, bool>();
	
	
	// -- Public Interface, identical to the ones provided by the default
	// unity GUI class.
	
	public static bool Button(Rect position, string text) {
		if (GUI.Button(position, text)) return true;
		return CheckButtonPressed(position, text, GUI.skin.button);
	}
	
	public static bool Button(Rect position, Texture image) {
		if (GUI.Button(position, image)) return true;
		return CheckButtonPressed(position, image, GUI.skin.button);
	}
	
	public static bool Button(Rect position, GUIContent content) {
		if (GUI.Button(position, content)) return true;
		return CheckButtonPressed(position, content, GUI.skin.button);
	}
	
	public static bool Button(Rect position, string text, GUIStyle style) {
		if (GUI.Button(position, text, style)) return true;
		return CheckButtonPressed(position, text, style);
	}
	
	public static bool Button(Rect position, Texture image, GUIStyle style) {
		if (GUI.Button(position, image, style)) return true;
		return CheckButtonPressed(position, image, style);
	}
	
	public static bool Button(Rect position, GUIContent content, GUIStyle style) {
		if (GUI.Button(position, content, style)) return true;
		return CheckButtonPressed(position, content, style);
	}

	public static bool RepeatButton(Rect position, string text) {
		if (GUI.RepeatButton(position, text)) return true;
		return CheckButtonDown(position);
	}
	
	public static bool RepeatButton(Rect position, Texture image) {
		if (GUI.RepeatButton(position, image)) return true;
		return CheckButtonDown(position);
	}
	
	public static bool RepeatButton(Rect position, GUIContent content) {
		if (GUI.Button(position, content)) return true;
		return CheckButtonDown(position);
	}
	
	public static bool RepeatButton(Rect position, string text, GUIStyle style) {
		if (GUI.RepeatButton(position, text, style)) return true;
		return CheckButtonDown(position);
	}
	
	public static bool RepeatButton(Rect position, Texture image, GUIStyle style) {
		if (GUI.RepeatButton(position, image, style)) return true;
		return CheckButtonDown(position);
	}
	
	public static bool RepeatButton(Rect position, GUIContent content, GUIStyle style) {
		if (GUI.RepeatButton(position, content, style)) return true;
		return CheckButtonDown(position);
	}
	
	// -- internal helper functions that do the actual work of mapping Leap data to UI
	// responses.
	
	static void CheckButtonHelper(Rect position, out bool pressed, out bool hovered) {
		if (m_leapController == null) {
			m_leapController = new Controller();
		}
		Frame frame = m_leapController.Frame();
		InteractionBox interactionBox = frame.InteractionBox;
		pressed = false;
		hovered = false;
		for(int i = 0; i < frame.Pointables.Count; ++i) {
			Vector3 screenPos = interactionBox.NormalizePoint(frame.Pointables[i].StabilizedTipPosition).ToUnity();
			float x = screenPos.x * UnityEngine.Screen.width;
			float y = (1 - screenPos.y) * UnityEngine.Screen.height;
			
			if (frame.Pointables[i].TouchZone == Pointable.Zone.ZONETOUCHING) {
				// this pointable is touching the screen, check to see if it's within the rect.
				if (x > position.x && x < position.xMax && y > position.y && y < position.yMax) {
					// down, now need to check to see if it was pressed in a previous frame.
					if (m_pressCache.ContainsKey(frame.Pointables[i].Id) == false) {
						pressed = true;
						m_pressCache.Add(frame.Pointables[i].Id, true);
					}
				}
			} else {
				if (x > position.x && x < position.xMax && y > position.y && y < position.yMax) {
					hovered = true;
				}
				m_pressCache.Remove(frame.Pointables[i].Id);
			}
		}
	}
	
	
	// returns true once during the first frame the button is detected as
	// pressed. Acts as a toggle, the same way Unity's default GUI.Button behaves.
	static bool CheckButtonPressed(Rect position, string text, GUIStyle style) {
		bool pressed;
		bool hovered;
		CheckButtonHelper(position, out pressed, out hovered);
		if (Event.current.type == EventType.Repaint) {
			style.Draw(position, text, hovered, pressed, false, false);
		}

		return pressed;
		
	}
	
	static bool CheckButtonPressed(Rect position, Texture image, GUIStyle style) {
		bool pressed;
		bool hovered;
		CheckButtonHelper(position, out pressed, out hovered);
		if (Event.current.type == EventType.Repaint) {
			style.Draw(position, image, hovered, pressed, false, false);
		}
		return pressed;
	}
	
	static bool CheckButtonPressed(Rect position, GUIContent content, GUIStyle style) {
		bool pressed;
		bool hovered;
		CheckButtonHelper(position, out pressed, out hovered);
		if (Event.current.type == EventType.Repaint) {
			style.Draw(position, content, hovered, pressed, false, false);
		}
		return pressed;
	}
	
	// returns true as long as the button is pressed.
	static bool CheckButtonDown(Rect position) {
		if (m_leapController == null) {
			m_leapController = new Controller();
		}
		Frame frame = m_leapController.Frame();
		InteractionBox interactionBox = frame.InteractionBox;
		for(int i = 0; i < frame.Pointables.Count; ++i) {
			if (frame.Pointables[i].TouchZone == Pointable.Zone.ZONETOUCHING) {
				// this pointable is touching the screen, check to see if it's within the rect.
				Vector3 screenPos = interactionBox.NormalizePoint(frame.Pointables[i].StabilizedTipPosition).ToUnity();
				float x = screenPos.x * UnityEngine.Screen.width;
				float y = (1 - screenPos.y) * UnityEngine.Screen.height;
				if (x > position.x && x < position.xMax && y > position.y && y < position.yMax) {
					return true;	
				}
			}
		}
		
		return false;
		
	}
	
}
